using System;
using System.IO;
using System.Xml.Serialization;

namespace protocol2html {
	public class Range {
		[XmlAttribute("type")]
		public string Type;
		
		[XmlAttribute("color")]
		public ushort Color;
		
		[XmlAttribute("start")]
		public ushort Start;
		
		[XmlAttribute("length")]
		public ushort Length;
		
		[XmlText(typeof(string))]
		public string Description;
	}

	public class Delimiter {
		[XmlAttribute("value")]
		public string Value;
	}

	public class PacketInfo {
		[XmlElement("bin",DataType="base64Binary")]
		public byte[] Bin;

		[XmlArray("ranges")]
		[XmlArrayItem("range")]
		public Range[] Ranges;

		[XmlArray("delimiters")]
		[XmlArrayItem("delimiter")]
		public Delimiter[] Delimiters;

		[XmlElement("comments")]
		public string Comments;
	}

	[XmlRoot("packet")]
	public class Packet {
		[XmlAttribute("name")]
		public string Name;

		[XmlElement("query")]
		public PacketInfo Query;

		[XmlElement("response")]
		public PacketInfo Response;
	}

	[XmlRoot("protocol")]
	public class Protocol {
		[XmlAttribute("name")]
		public string Name;

		[XmlElement("packet")]
		public Packet[] Packets;

		public static Protocol FromFile(string file) {
			FileStream fs=new FileStream(file, FileMode.Open, FileAccess.Read);
			XmlSerializer xs=new XmlSerializer(typeof(Protocol));
			Protocol p=(Protocol)xs.Deserialize(fs);
			fs.Close();

			return p;
		}
	}
}
